@@ -65,6 +65,13 @@
 #include <string.h>
 #include <unistd.h>
 
+#ifdef ENABLE_NLS
+#include <libintl.h>
+#define _(x) dgettext(PACKAGE_NAME, x)
+#else
+#define _(x) (x)
+#endif
+
 struct _p11_parser {
 	p11_asn1_cache *asn1_cache;
 	p11_dict *asn1_defs;
@@ -99,7 +106,7 @@ populate_trust (p11_parser *parser,
 	 */
 	if (parser->flags & P11_PARSE_FLAG_ANCHOR) {
 		if (p11_attrs_find_bool (attrs, CKA_X_DISTRUSTED, &distrustv) && distrustv) {
-			p11_message ("certificate with distrust in location for anchors: %s", parser->basename);
+			p11_message (_("certificate with distrust in location for anchors: %s"), parser->basename);
 			return attrs;
 
 		}
@@ -108,19 +115,19 @@ populate_trust (p11_parser *parser,
 		distrustv = CK_FALSE;
 
 	/*
-	 * If we're parsing a blacklist location, then force all certificates to
-	 * be blacklisted, regardless of whether they contain anchor information.
+	 * If we're parsing a blocklist location, then force all certificates to
+	 * be distrusted, regardless of whether they contain anchor information.
 	 */
-	} else if (parser->flags & P11_PARSE_FLAG_BLACKLIST) {
+	} else if (parser->flags & P11_PARSE_FLAG_BLOCKLIST) {
 		if (p11_attrs_find_bool (attrs, CKA_TRUSTED, &trustedv) && trustedv)
-			p11_message ("overriding trust for anchor in blacklist: %s", parser->basename);
+			p11_message (_("overriding trust for anchor in blocklist: %s"), parser->basename);
 
 		trustedv = CK_FALSE;
 		distrustv = CK_TRUE;
 
 	/*
 	 * If the location doesn't have a flag, then fill in trust attributes
-	 * if they are missing: neither an anchor or blacklist.
+	 * if they are missing: neither an anchor or blocklist.
 	 */
 	} else {
 		trustedv = CK_FALSE;
@@ -432,9 +439,9 @@ build_openssl_extensions (p11_parser *parser,
 	}
 
 	/*
-	 * OpenSSL model blacklists as anchors with all purposes being removed/rejected,
+	 * OpenSSL model blocklists as anchors with all purposes being removed/rejected,
 	 * we account for that here. If there is an ExtendedKeyUsage without any
-	 * useful purposes, then treat like a blacklist.
+	 * useful purposes, then treat like a blocklist.
 	 */
 	if (trust && p11_dict_size (trust) == 0) {
 		trusted = CK_FALSE;
@@ -450,9 +457,9 @@ build_openssl_extensions (p11_parser *parser,
 	}
 
 	/*
-	 * OpenSSL model blacklists as anchors with all purposes being removed/rejected,
+	 * OpenSSL model blocklists as anchors with all purposes being removed/rejected,
 	 * we account for that here. If there is an ExtendedKeyUsage without any
-	 * useful purposes, then treat like a blacklist.
+	 * useful purposes, then treat like a blocklist.
 	 */
 
 	cert = p11_attrs_merge (cert, p11_attrs_dup (trust_attrs), true);
@@ -586,7 +593,7 @@ on_pem_block (const char *type,
 	}
 
 	if (ret != P11_PARSE_SUCCESS)
-		p11_message ("Couldn't parse PEM block of type %s", type);
+		p11_message (_("Couldn't parse PEM block of type %s"), type);
 }
 
 int
@@ -756,7 +763,7 @@ p11_parse_file (p11_parser *parser,
 
 	map = p11_mmap_open (filename, sb, &data, &size);
 	if (map == NULL) {
-		p11_message_err (errno, "couldn't open and map file: %s", filename);
+		p11_message_err (errno, _("couldn't open and map file: %s"), filename);
 		return P11_PARSE_FAILURE;
 	}
 
