@@ -54,6 +54,9 @@
 
 #include <sys/stat.h>
 #include <sys/types.h>
+#ifdef OS_UNIX
+#include <unistd.h>
+#endif
 
 #include <assert.h>
 #include <dirent.h>
@@ -61,6 +64,13 @@
 #include <stdlib.h>
 #include <string.h>
 
+#ifdef ENABLE_NLS
+#include <libintl.h>
+#define _(x) dgettext(PACKAGE_NAME, x)
+#else
+#define _(x) (x)
+#endif
+
 struct _p11_token {
 	p11_parser *parser;       /* Parser we use to load files */
 	p11_index *index;         /* Index we load objects into */
@@ -69,7 +79,7 @@ struct _p11_token {
 
 	char *path;               /* Main path to load from */
 	char *anchors;            /* Path to load anchors from */
-	char *blacklist;          /* Path to load blacklist from */
+	char *blocklist;          /* Path to load blocklist from */
 	char *label;              /* The token label */
 	CK_SLOT_ID slot;          /* The slot id */
 
@@ -175,9 +185,9 @@ loader_load_file (p11_token *token,
 	if (p11_path_prefix (filename, token->anchors))
 		flags = P11_PARSE_FLAG_ANCHOR;
 
-	/* If it's in the blacklist subdirectory, treat as a blacklist */
-	else if (p11_path_prefix (filename, token->blacklist))
-		flags = P11_PARSE_FLAG_BLACKLIST;
+	/* If it's in the blocklist subdirectory, treat as a blocklist */
+	else if (p11_path_prefix (filename, token->blocklist))
+		flags = P11_PARSE_FLAG_BLOCKLIST;
 
 	/* If the token is just one path, then assume they are anchors */
 	else if (strcmp (filename, token->path) == 0 && !S_ISDIR (sb->st_mode))
@@ -214,7 +224,7 @@ loader_load_file (p11_token *token,
 	p11_index_finish (token->index);
 
 	if (rv != CKR_OK) {
-		p11_message ("couldn't load file into objects: %s", filename);
+		p11_message (_("couldn't load file into objects: %s"), filename);
 		return -1;
 	}
 
@@ -230,7 +240,7 @@ loader_load_if_file (p11_token *token,
 
 	if (stat (path, &sb) < 0) {
 		if (errno != ENOENT)
-			p11_message_err (errno, "couldn't stat path: %d: %s", errno, path);
+			p11_message_err (errno, _("couldn't stat path: %d: %s"), errno, path);
 
 	} else if (!S_ISDIR (sb.st_mode)) {
 		return loader_load_file (token, path, &sb);
@@ -256,7 +266,7 @@ loader_load_directory (p11_token *token,
 	/* First we load all the modules */
 	dir = opendir (directory);
 	if (!dir) {
-		p11_message_err (errno, "couldn't list directory: %s", directory);
+		p11_message_err (errno, _("couldn't list directory: %s"), directory);
 		loader_not_loaded (token, directory);
 		return 0;
 	}
@@ -299,7 +309,7 @@ loader_load_path (p11_token *token,
 
 	if (stat (path, &sb) < 0) {
 		if (errno != ENOENT)
-			p11_message_err (errno, "cannot access trust certificate path: %s", path);
+			p11_message_err (errno, _("cannot access trust certificate path: %s"), path);
 		loader_gone_file (token, path);
 		*is_dir = false;
 		ret = 0;
@@ -385,7 +395,7 @@ p11_token_load (p11_token *token)
 		if (ret >= 0)
 			total += ret;
 
-		ret = loader_load_path (token, token->blacklist, &is_dir);
+		ret = loader_load_path (token, token->blocklist, &is_dir);
 		if (ret >= 0)
 			total += ret;
 	}
@@ -413,7 +423,7 @@ p11_token_reload (p11_token *token,
 		if (errno == ENOENT) {
 			loader_gone_file (token, origin);
 		} else {
-			p11_message_err (errno, "cannot access trust file: %s", origin);
+			p11_message_err (errno, _("cannot access trust file: %s"), origin);
 		}
 		ret = false;
 
@@ -462,7 +472,7 @@ check_directory (const char *path,
 		free (parent);
 		return ret;
 	default:
-		p11_message_err (errno, "couldn't access: %s", path);
+		p11_message_err (errno, _("couldn't access: %s"), path);
 		return false;
 	}
 }
@@ -490,7 +500,7 @@ writer_remove_origin (p11_token *token,
 	return_val_if_fail (path != NULL, false);
 
 	if (unlink (path) < 0) {
-		p11_message_err (errno, "couldn't remove file: %s", path);
+		p11_message_err (errno, _("couldn't remove file: %s"), path);
 		ret = false;
 	}
 
@@ -622,7 +632,7 @@ mkdir_with_parents (const char *path)
 		}
 		/* fall through */
 	default:
-		p11_message_err (errno, "couldn't create directory: %s", path);
+		p11_message_err (errno, _("couldn't create directory: %s"), path);
 		return false;
 	}
 }
@@ -809,7 +819,7 @@ p11_token_free (p11_token *token)
 	p11_dict_free (token->loaded);
 	free (token->path);
 	free (token->anchors);
-	free (token->blacklist);
+	free (token->blocklist);
 	free (token->label);
 	free (token);
 }
@@ -855,8 +865,8 @@ p11_token_new (CK_SLOT_ID slot,
 	token->anchors = p11_path_build (token->path, "anchors", NULL);
 	return_val_if_fail (token->anchors != NULL, NULL);
 
-	token->blacklist = p11_path_build (token->path, "blacklist", NULL);
-	return_val_if_fail (token->blacklist != NULL, NULL);
+	token->blocklist = p11_path_build (token->path, "blocklist", NULL);
+	return_val_if_fail (token->blocklist != NULL, NULL);
 
 	token->label = strdup (label);
 	return_val_if_fail (token->label != NULL, NULL);
